% Copyright 2018. All Rights Reserved
% Code by Shuang Li
% For Paper, "Atomic Norm Minimization for Modal Analysis from Random and Compressed Samples"
% IEEE transactions on signal processing, 2018
% by Shuang Li, Dehui Yang, Gongguo Tang, and Michael B. Wakin
% This code is used to reproduce Figure 2.

%% Generate data matrix  
clc; clear; close all;

m = 1:6;  
M = diag(m);   % mass matrix
k1 = 500; k2 = 150; k3 = 100; k4 = 50; k5 = 100; k6 = 150; k7 = 500;
K = [k1+k2 -k2 0 0 0 0; -k2 k2+k3 -k3 0 0 0; 0 -k3 k3+k4 -k4 0 0; 0 0 -k4 k4+k5 -k5 0;...
    0 0 0 -k5 k5+k6 -k6; 0 0 0 0 -k6 k6+k7];  % stiffness matrix
[mshape, S] = eig(K,M);
mshape = mshape*diag(1./norms(mshape));   % mode shapes
Freq = sqrt(diag(S)); 
freq = sort(Freq/2/pi);    % frequencies in Hz
A = [1 0.85 0.7 0.5 0.25 0.1];  % amplitudes

Nsensor=size(mshape,1);   % number of sensor
Nshape=size(mshape,2);    % number of active modes
Nsample=100;              % number of samples
Ts=0.9/max(freq);         % sampling interval
Time=0:Ts:Ts*(Nsample-1); % sampling time
QQ = exp(1j*2*pi*freq*Time);
Data=mshape*diag(A)*QQ;
Data= transpose(Data);    % generated data matrix

%% Atomic Norm Minimization
cvx_begin quiet
    variable u0;
    variable u(Nsample-1) complex;
    variable Tu(Nsample,Nsample) hermitian;
    variable X(Nsample,Nsensor) complex;
    variable W(Nsensor,Nsensor) Hermitian; 
    dual variable Y0 
    minimize(0.5/Nsample*trace(Tu)+0.5*trace(W))
    subject to 
    Tu == toeplitz([u0;conj(u)],[u0;conj(u)]');
    Y0:[Tu   X; ...
        X'   W] == semidefinite(Nsample+Nsensor,Nsample+Nsensor);
    X==Data;
cvx_end

Y=-2*Y0(1:Nsample,Nsample+1:Nsample+Nsensor);  % dual solution  
f=0:0.0001:5;           % frequency grid
a=exp(1j*2*pi*Time'*f); % atoms
Q=Y'*a;                 % dual polynomial
if Nsensor > 1
    NormQ=norms(Q);   % norm of dual polynomial
else
    NormQ=abs(Q);
end

%% Identify frequencies by finding places where the norm of dual polynomial achieves 1.
figure
lw=2;      % linewidth
fs=29;     % FontSize
hold on;
plot(f,NormQ,'b-','Linewidth',lw);
[pks,locs] = findpeaks(NormQ,'minpeakdistance',1,'MinPeakHeight',0.99);
stem(f(locs),pks,'','r^','markerfacecolor',[1 0 0],'Linewidth',lw)
freq_true = freq'  % true frequencies
freq_est=f(locs)   % estimated frequencies
xlabel('Frequency(Hz)','FontSize',fs);
ylabel('$||\mathcal{Q}(f)||_2$','Interpreter','LaTex','FontSize',fs);
set(gca,'FontSize',fs);

%% 
Af=exp(1j*2*pi*Time'*freq_est);
Phi=Af\X;  
Phi=transpose(Phi);
A_est=norms(Phi);    % estimated coefficients 
MS_estimate = Phi*diag(1./A_est);  % estimated mode shapes

MAC_ANM = diag(abs(mshape'*MS_estimate)) % compute MAC

%% Plot the true frequencies and the estimated frequencies
figure
lw=4;      % linewidth
fs=29;     % FontSize
ms=18;
hold on;
stem(freq,A,'rx','Linewidth',lw,'Markersize',ms);
stem(freq_est,A_est,'bo','Linewidth',lw,'Markersize',ms);
legend('True Frequencies','Estimated Frequencies' );
xlabel('Frequency(Hz)','FontSize',fs);
ylabel('Amplitude','FontSize',fs);
set(gca,'FontSize',fs);

%% Find mode shapes via SVD
[u,s,~] = svd(transpose(Data));  % !!!!!!
MShape_SVD=u;  
MAC_SVD=abs(mshape'*MShape_SVD);
idx1=find(MAC_SVD(1,:)==max(MAC_SVD(1,:)));
idx2=find(MAC_SVD(2,:)==max(MAC_SVD(2,:)));
idx3=find(MAC_SVD(3,:)==max(MAC_SVD(3,:)));
idx4=find(MAC_SVD(4,:)==max(MAC_SVD(4,:)));
idx5=find(MAC_SVD(5,:)==max(MAC_SVD(5,:)));
idx6=find(MAC_SVD(6,:)==max(MAC_SVD(6,:)));
MShape_SVD=MShape_SVD(:,[idx1 idx2 idx3 idx4 idx5 idx6]); % recovered mode shapes
MAC_SVD=diag(abs(mshape'*MShape_SVD))  % compute MAC

%% Plot true mode shapes and mode shapes recovered with ANM and SVD)
figure
lw=6;
fs=29;
subplot(3,2,1)
plot(real(mshape(:,1)),'r-','Linewidth',lw);hold on;
plot(real(MS_estimate(:,1)),'b--','Linewidth',lw);hold on;
plot(real(MShape_SVD(:,1)),'g-.','Linewidth',lw);hold on;
ylabel('Mode Shape 1','FontSize',fs)
set(gca,'FontSize',fs);
axis tight;

subplot(3,2,2)
plot(real(mshape(:,2)),'r-','Linewidth',lw);hold on;
plot(real(MS_estimate(:,2)),'b--','Linewidth',lw);hold on;
plot(real(-MShape_SVD(:,2)),'g-.','Linewidth',lw);hold on;
ylabel('Mode Shape 2','FontSize',fs)
legend('True', 'ANM', 'SVD')
set(gca,'FontSize',fs);
axis tight;

subplot(3,2,3)
plot(real(mshape(:,3)),'r-','Linewidth',lw);hold on;
plot(real(MS_estimate(:,3)),'b--','Linewidth',lw);hold on;
plot(real(MShape_SVD(:,3)),'g-.','Linewidth',lw);hold on;
ylabel('Mode Shape 3','FontSize',fs)
set(gca,'FontSize',fs);
axis tight;


subplot(3,2,4)
plot(real(mshape(:,4)),'r-','Linewidth',lw);hold on;
plot(real(MS_estimate(:,4)),'b--','Linewidth',lw);hold on;
plot(real(MShape_SVD(:,4)),'g-.','Linewidth',lw);hold on;
ylabel('Mode Shape 4','FontSize',fs)
set(gca,'FontSize',fs);
axis tight;

subplot(3,2,5)
plot(real(mshape(:,5)),'r-','Linewidth',lw);hold on;
plot(real(MS_estimate(:,5)),'b--','Linewidth',lw);hold on;
plot(real(MShape_SVD(:,5)),'g-.','Linewidth',lw);hold on;
xlabel('Node','FontSize',fs)
ylabel('Mode Shape 5','FontSize',fs)
set(gca,'FontSize',fs);
axis tight;

subplot(3,2,6)
plot(real(mshape(:,6)),'r-','Linewidth',lw);hold on;
plot(real(MS_estimate(:,6)),'b--','Linewidth',lw);hold on;
plot(real(-MShape_SVD(:,6)),'g-.','Linewidth',lw);hold on;
xlabel('Node','FontSize',fs)
ylabel('Mode Shape 6','FontSize',fs)
set(gca,'FontSize',fs);
axis tight;


MinSep = (freq(5)-freq(4))*Ts   % minimum separation



